;***********************************************************************
;
;	TSIC-zacwire-Protokoll am PIC
;
;***********************************************************************
; Pinbelegung PIC16F886
;
;                  +---###---+
;   Vpp/input RE 3 |  1   28 | RB 7 PGD
;             RA 0 |  2   27 | RB 6 PGC
;             RA 1 |  3   26 | RB 5
;             RA 2 |  4   25 | RB 4
;             RA 3 |  5   24 | RB 3
;             RA 4 |  6   23 | RB 2
;             RA 5 |  7   22 | RB 1 / LED 1
;          --- Vss |  8   21 | RB 0 / LED 0
;      OSC1 / RA 7 |  9   20 | Vdd +++
;      OSC2 / RA 6 | 10   19 | Vss ---
; Qrz T1OSC / RC 0 | 11   18 | RC 7 / RX
; Qrz T1OSC / RC 1 | 12   17 | RC 6 / TX / RSOUTPIN
;             RC 2 | 13   16 | RC 5 / WRITE_LO
;       SCL / RC 3 | 14   15 | RC 4 / SDA
;                  +---------+
;
;***********************************************************************
; Definition C, Programm luft auch mit 16F882 und 16F883, vermutlich auch 16F887 (nicht getestet)

	list p=16f886
	#include <p16f886.inc>

	ERRORLEVEL -302				; schaltet Bank-Warnungen ab

; CONFIG 16F886

	__CONFIG    _CONFIG1, _DEBUG_OFF & _CP_OFF & _LVP_OFF & _BOR_OFF & _MCLRE_OFF & _PWRTE_ON & _WDT_OFF & _INTOSCIO
	__CONFIG    _CONFIG2, _WRT_OFF & _BOR21V 

;***********************************************************************
; Variablen festlegen  
; 0x020..0x06F
; 0x0A0..0x0EF
; 0x120..0x16F 
; 0x1A0..0x1EF
; 0x070..0x07F = 0x0F0..0x0FF = 0x170..0x17F = 0x1F0..0x1FF (also in allen Banks)

	cblock 0x70

	w_temp						; Interrupt Zwischenspeicher
	s_temp						;
	p_temp						;

	endc

	cblock 0x20

	_t_strobe					; Timerstartwert (255 - t_strobe)

	bitcount					; zhlt die Datenbits (z.B. RS232)
	d_byte						; Zwischenspeicher fr Datenbyte (z.B. RS232)

	te0_lo						; Daten von TSIC #0
	te0_hi						;
	te1_lo						; Daten von TSIC #1
	te1_hi						;

	flags1						; Flags

	endc

#define parity		flags1, 0	; Parity-Bits

#define POWER		PORTC, 2	; Power fr TSIC

#define TSIC_0		PORTA, 0	; Eingangs-Port fr TSIC #0
#define TSIC_1		PORTA, 1	; Eingangs-Port fr TSIC #1

bank0 macro
	bcf		STATUS, RP0			; Bank 0
	bcf		STATUS, RP1
	endm

bank1 macro
	bsf		STATUS, RP0			; Bank 1
	bcf		STATUS, RP1
	endm

bank2 macro
	bcf		STATUS, RP0			; Bank 2
	bsf		STATUS, RP1
	endm

bank3 macro
	bsf		STATUS, RP0			; Bank 3
	bsf		STATUS, RP1
	endm



;***********************************************************************
;***********************************************************************

	org		0x00

	goto	init

;***********************************************************************
;***********************************************************************
; Interrupt

; hier nix Interrupt

;*********************************************************************** 
;***********************************************************************
; Initialisierung

init

; Ports

	bank3
	clrf	ANSEL				; Port A digital
	clrf	ANSELH				; Port B digital

	bank1
	clrf	TRISA				; Port A alle output
	clrf	TRISB				; Port B alle output
	clrf	TRISC				; Port C alle output
	clrf	TRISE				; Port E alle output

	bsf		TRISA, 0			; RA0 input fr TSIC Daten

	bank0
	clrf	PORTA				; Ports initialisieren
	clrf	PORTB
	clrf	PORTC
	clrf	PORTE

; internen Oscillator einstellen, falls konfiguriert

	bank1
	movlw	B'01100001'			; 4 MHz intern
	movwf	OSCCON
; OSCTUNE geht nicht beim 31.25-kHz-Oszillator !
	movlw	B'00000000'
	movwf	OSCTUNE				; Osc Mittelfrequenz

; Timer 0 einstellen

	bank1
	bcf		OPTION_REG, T0CS	; Takt intern
	bsf		OPTION_REG, PSA		; Prescaler fr WDT, Timer 0 also ohne Prescaler, also ca 60 Takte fr 1/2 Datenbit

;	bcf		OPTION_REG, PS2		; 000 = prescaler=1/2, bei 4 MHz 2 us -> 500kHz
;	bcf		OPTION_REG, PS1		; das sind ca 30 Takte fr 1/2 Datenbit = t_strobe
;	bcf		OPTION_REG, PS0		;

; verschiedenes

	bank0



;***********************************************************************
;***********************************************************************

main

	...
	...
	...

;***********************************************************************
; mit TSIC messen

messen

; Daten vom TSIC holen

; einen Wert = 2 Byte von TSIC #0 messen und in te0_lo / te0_hi ablegen
; es fehlt bei getbyte0 noch Test auf Timeout

	bsf		POWER				; schalte TSIC ein
	call	wait100ms			; 140 ms braucht der TSIC zum hochfahren, Wartezeit muss etwas kleiner sein

	call	getbyte0			; HI-Datenbyte lesen
	movwf	te0_hi
	call	getbyte0			; LO-Datenbyte lesen
	movwf	te0_lo

	bcf		POWER				; schalte TSIC aus

	...
	...
	...



;***********************************************************************
;***********************************************************************
;***********************************************************************
; TSIC #0: hole Startbit, 8 Datenbits (MSB..LSB), Paritybit
; D0 = Bit fr TSIC Data
;
; Duty-Cycle:
; Startbit                   50%
; Daten-/Parity-Bit logic 0: 25%
;                   logic 1: 75%
; typ. Lnge eines Bits: 125 usec
; Timer 0 luft mit 1 usec -> ca 125 Takte fr 1 Bit, ca 60 Takte fr 1/2 Bit -> t_strobe
;
; es fehlt noch Test auf timeout

; Messprinzip:
; Die Bitlnge ist temperaturabhngig, deshalb bestimmt man die Bitlnge bei jedem Byte durch Messung des Startbits
; dieses hat einen Duty-Cycle von 50%. Wenn das halbe Startbit rum ist (LO), wird die Zeit t_strobe gemessen.
; Diese Zeit wartet man vom Start eines jeden Daten-Bytes, und nimmt dann den Zustand des Daten-Pins auf. Ist er
; LO, dann ist das Daten-Bit 0 (25% Duty-Cycle), ist er HI, dann ist das Daten-Bit 1 (75% Duty-Cycle).

getbyte0

; Startbit lesen (Zeit bestimmen, die halbes Bit braucht = t_strobe)

	btfsc	TSIC_0				; TSIC #0 Data LO = Beginn Startbit
	goto	$-1					; warten und warten und warten ...

	bcf		INTCON, TMR0IF		; Timer 0 flag bit rcksetzen
	clrf	TMR0				; Timer 0 rcksetzen

	btfss	TSIC_0				; TSIC #0 Data HI = t_strobe beendet
	goto	$-1					;

	movfw	TMR0				; hole t_strobe vom Timer 0
	sublw	0xFF				; von 255 abziehen = Timerstartwert
	movwf	_t_strobe			;

; 8 Datenbits lesen

	movlw	0x08				; Anzahl Datenbits
	movwf	bitcount			;

datenbit0

; Start Datenbit = HI -> LO

	btfsc	TSIC_0				; TSIC #0 Data LO = Beginn Datenbit
	goto	$-1

	bcf		INTCON, TMR0IF		; Timer 0 flag bit rcksetzen
	movfw	_t_strobe			; Startwert
	movwf	TMR0				; Timer 0 vorbesetzen und starten

	btfss	INTCON, TMR0IF		; warten, dass Timer 0 bergelaufen 
	goto	$-1					;

; schiebe Datenbit ber Carry von unten ins Datenbyte (MSB first)

	btfss	TSIC_0				; Datenbit TSIC #0 gesetzt ?
	goto 	$+3
	bsf		STATUS, C			; ja, set Carry
	goto	$+2
	bcf		STATUS, C			; nein, clear Carry
	rlf		d_byte, F			; Carry in Bit 0 des Datenbytes schieben (MSB first)

	btfss	TSIC_0				; TSIC #0 Data wieder hi ?
	goto	$-1					;

	decfsz	bitcount, F			; alle Bits gelesen ?
	goto	datenbit0			; nein, dann weitere Bits lesen

; Paritybit lesen (wird aber nicht ausgewertet)

	btfsc	TSIC_0				; TSIC #0 Data LO = Beginn Paritybit
	goto	$-1					;

	bcf		INTCON, TMR0IF		; Timer 0 flag bit rcksetzen
	movfw	_t_strobe			; Startwert
	movwf	TMR0				; Timer 0 vorbesetzen und starten

	btfss	INTCON, TMR0IF		; warten, dass Timer 0 bergelaufen
	goto	$-1					;

	btfss	TSIC_0				; Paritybit TSIC #0 gesetzt ?
	goto 	$+3
	bsf		parity				; ja, set Parity
	goto	$+2
	bcf		parity				; nein, clear Parity

	btfss	TSIC_0				; TSIC #0 Data wieder hi ?
	goto	$-1					;

; Datenbyte nach W

	movfw	d_byte				; Datenbyte nach W

	return



;***********************************************************************
;***********************************************************************

	end

;***********************************************************************
